/*##############################################################################################*
*###############################################################################################*
*                                                                                               *
*                                               TCPIP.H                                         *
*                                       Header file of MpTcp                                    *
*                                                                                               *
*                                              -- ## --                                         *
*                                          Mptcp Ver: 1.6                                       *
*                                  (This file is part of MpTcp tool)                            *
*                                       Date: 10/02/2011 (11:15)                                *
*                                                                                               *
*                                                                                               *
*                       .......................................................                 *
*                                                                                               *
*                                Felipe de Oliveira - khun@hexcodes.org                         *
*                                                                                               *
*                                                                                               *
###############################################################################################*/


#ifndef TCPIP_H
#define TCPIP_H

#include <features.h>
#include <netinet/in.h>
#include <sys/cdefs.h>
#include <sys/types.h>
#include <linux/if_ether.h> 

// ---------------------------- ## ---------------------------

struct iphdr
  {
#if __BYTE_ORDER == __LITTLE_ENDIAN
    unsigned int ihl:4;
    unsigned int version:4;
#elif __BYTE_ORDER == __BIG_ENDIAN
    unsigned int version:4;
    unsigned int ihl:4;
#endif
    u_int8_t tos;
    u_int16_t tot_len;
    u_int16_t id;
    u_int16_t frag_off;
    u_int8_t ttl;
    u_int8_t protocol;
    u_int16_t check;
    u_int32_t saddr;
    u_int32_t daddr;
  };

struct ip
  {
#if __BYTE_ORDER == __LITTLE_ENDIAN
    unsigned int ip_hl:4;		
    unsigned int ip_v:4;	
#endif
#if __BYTE_ORDER == __BIG_ENDIAN
    unsigned int ip_v:4;		
    unsigned int ip_hl:4;	
#endif
    u_int8_t ip_tos;	
    u_short ip_len;
    u_short ip_id;
    u_short ip_off;			
#define	IP_RF 0x8000		
#define	IP_DF 0x4000	
#define	IP_MF 0x2000
#define	IP_OFFMASK 0x1fff		
    u_int8_t ip_ttl;		
    u_int8_t ip_p;	
    u_short ip_sum;			
    struct in_addr ip_src, ip_dst;
  };

// ============================= ## ==================================


#define	TCP_NODELAY	 1	
#define	TCP_MAXSEG	 2
#define TCP_CORK	 3
#define TCP_KEEPIDLE	 4	
#define TCP_KEEPINTVL	 5	
#define TCP_KEEPCNT	 6	
#define TCP_SYNCNT	 7	
#define TCP_LINGER2	 8	
#define TCP_DEFER_ACCEPT 9	
#define TCP_WINDOW_CLAMP 10	
#define TCP_INFO	 11	
#define	TCP_QUICKACK	 12	

# ifdef __FAVOR_BSD
typedef	u_int32_t tcp_seq;
struct tcphdr
  {
    u_int16_t th_sport;		
    u_int16_t th_dport;		
    tcp_seq th_seq;		
    tcp_seq th_ack;		
#  if __BYTE_ORDER == __LITTLE_ENDIAN
    u_int8_t th_x2:4;	
    u_int8_t th_off:4;	
#  endif
#  if __BYTE_ORDER == __BIG_ENDIAN
    u_int8_t th_off:4;		
    u_int8_t th_x2:4;	
#  endif
    u_int8_t th_flags;
#  define TH_FIN	0x01
#  define TH_SYN	0x02
#  define TH_RST	0x04
#  define TH_PUSH	0x08
#  define TH_ACK	0x10
#  define TH_URG	0x20
    u_int16_t th_win;		
    u_int16_t th_sum;		
    u_int16_t th_urp;		
};
# else /* Se NAO BSD - IF NOT __FAVOR_BSD */

struct tcphdr
  {
    u_int16_t source;
    u_int16_t dest;
    u_int32_t seq;
    u_int32_t ack_seq;
#  if __BYTE_ORDER == __LITTLE_ENDIAN
    u_int16_t res1:4;
    u_int16_t doff:4;
    u_int16_t fin:1;
    u_int16_t syn:1;
    u_int16_t rst:1;
    u_int16_t psh:1;
    u_int16_t ack:1;
    u_int16_t urg:1;
    u_int16_t res2:2;
#  elif __BYTE_ORDER == __BIG_ENDIAN
    u_int16_t doff:4;
    u_int16_t res1:4;
    u_int16_t res2:2;
    u_int16_t urg:1;
    u_int16_t ack:1;
    u_int16_t psh:1;
    u_int16_t rst:1;
    u_int16_t syn:1;
    u_int16_t fin:1;
#  else
#   error "Ajuste Bitse -  Adjust Bits at <bits/endian.h> defines"
#  endif  // PARA BIT ORDER...
    u_int16_t window;
    u_int16_t check;
    u_int16_t urg_ptr;
};
# endif  // __FAVOR_BSD

enum
{
  TCP_ESTABLISHED = 1,
  TCP_SYN_SENT,
  TCP_SYN_RECV,
  TCP_FIN_WAIT1,
  TCP_FIN_WAIT2,
  TCP_TIME_WAIT,
  TCP_CLOSE,
  TCP_CLOSE_WAIT,
  TCP_LAST_ACK,
  TCP_LISTEN,
  TCP_CLOSING   
};

# define TCPOPT_EOL		0
# define TCPOPT_NOP		1
# define TCPOPT_MAXSEG		2
# define TCPOLEN_MAXSEG		4
# define TCPOPT_WINDOW		3
# define TCPOLEN_WINDOW		3
# define TCPOPT_SACK_PERMITTED	4		
# define TCPOLEN_SACK_PERMITTED	2
# define TCPOPT_SACK		5	
# define TCPOPT_TIMESTAMP	8
# define TCPOLEN_TIMESTAMP	10
# define TCPOLEN_TSTAMP_APPA	(TCPOLEN_TIMESTAMP+2) 

# define TCPOPT_TSTAMP_HDR	\
    (TCPOPT_NOP<<24|TCPOPT_NOP<<16|TCPOPT_TIMESTAMP<<8|TCPOLEN_TIMESTAMP)

# define TCP_MSS	512
# define TCP_MAXWIN	65535	
# define TCP_MAX_WINSHIFT	14	
# define SOL_TCP		6
# define TCPI_OPT_TIMESTAMPS	1
# define TCPI_OPT_SACK		2
# define TCPI_OPT_WSCALE	4
# define TCPI_OPT_ECN		8

enum tcp_ca_state
{
  TCP_CA_Open = 0,
  TCP_CA_Disorder = 1,
  TCP_CA_CWR = 2,
  TCP_CA_Recovery = 3,
  TCP_CA_Loss = 4
};

struct tcp_info
{
  u_int8_t	tcpi_state;
  u_int8_t	tcpi_ca_state;
  u_int8_t	tcpi_retransmits;
  u_int8_t	tcpi_probes;
  u_int8_t	tcpi_backoff;
  u_int8_t	tcpi_options;
  u_int8_t	tcpi_snd_wscale : 4, tcpi_rcv_wscale : 4;

  u_int32_t	tcpi_rto;
  u_int32_t	tcpi_ato;
  u_int32_t	tcpi_snd_mss;
  u_int32_t	tcpi_rcv_mss;

  u_int32_t	tcpi_unacked;
  u_int32_t	tcpi_sacked;
  u_int32_t	tcpi_lost;
  u_int32_t	tcpi_retrans;
  u_int32_t	tcpi_fackets;


  u_int32_t	tcpi_last_data_sent;
  u_int32_t	tcpi_last_ack_sent;
  u_int32_t	tcpi_last_data_recv;
  u_int32_t	tcpi_last_ack_recv;


  u_int32_t	tcpi_pmtu;
  u_int32_t	tcpi_rcv_ssthresh;
  u_int32_t	tcpi_rtt;
  u_int32_t	tcpi_rttvar;
  u_int32_t	tcpi_snd_ssthresh;
  u_int32_t	tcpi_snd_cwnd;
  u_int32_t	tcpi_advmss;
  u_int32_t	tcpi_reordering;
};

//=================================== ## ==========================


struct icmphdr
{
  u_int8_t type;		
  u_int8_t code;		
  u_int16_t checksum;
  union
  {
    struct
    {
      u_int16_t	id;
      u_int16_t	sequence;
    } echo;		
    u_int32_t	gateway;
    struct
    {
      u_int16_t	__unused;
      u_int16_t	mtu;
    } frag;	
  } un;
};

#define ICMP_ECHOREPLY		0	
#define ICMP_DEST_UNREACH	3	
#define ICMP_SOURCE_QUENCH	4	
#define ICMP_REDIRECT		5	
#define ICMP_ECHO		8	
#define ICMP_TIME_EXCEEDED	11	
#define ICMP_PARAMETERPROB	12	
#define ICMP_TIMESTAMP		13	
#define ICMP_TIMESTAMPREPLY	14	
#define ICMP_INFO_REQUEST	15
#define ICMP_INFO_REPLY		16	
#define ICMP_ADDRESS		17
#define ICMP_ADDRESSREPLY	18	
#define NR_ICMP_TYPES		18

#define ICMP_NET_UNREACH	0	
#define ICMP_HOST_UNREACH	1	
#define ICMP_PROT_UNREACH	2	
#define ICMP_PORT_UNREACH	3	
#define ICMP_FRAG_NEEDED	4	
#define ICMP_SR_FAILED		5	
#define ICMP_NET_UNKNOWN	6
#define ICMP_HOST_UNKNOWN	7
#define ICMP_HOST_ISOLATED	8
#define ICMP_NET_ANO		9
#define ICMP_HOST_ANO		10
#define ICMP_NET_UNR_TOS	11
#define ICMP_HOST_UNR_TOS	12
#define ICMP_PKT_FILTERED	13	
#define ICMP_PREC_VIOLATION	14	
#define ICMP_PREC_CUTOFF	15	
#define NR_ICMP_UNREACH		15	

#define ICMP_REDIR_NET		0	
#define ICMP_REDIR_HOST		1	
#define ICMP_REDIR_NETTOS	2	
#define ICMP_REDIR_HOSTTOS	3	

#define ICMP_EXC_TTL		0	
#define ICMP_EXC_FRAGTIME	1	


struct icmp_ra_addr
{
  u_int32_t ira_addr;
  u_int32_t ira_preference;
};

struct icmp
{
  u_int8_t  icmp_type;	
  u_int8_t  icmp_code;	
  u_int16_t icmp_cksum;	
  union
  {
    u_char ih_pptr;		
    struct in_addr ih_gwaddr;	
    struct ih_idseq		
    {
      u_int16_t icd_id;
      u_int16_t icd_seq;
    } ih_idseq;
    u_int32_t ih_void;

    struct ih_pmtu
    {
      u_int16_t ipm_void;
      u_int16_t ipm_nextmtu;
    } ih_pmtu;

    struct ih_rtradv
    {
      u_int8_t irt_num_addrs;
      u_int8_t irt_wpa;
      u_int16_t irt_lifetime;
    } ih_rtradv;
  } icmp_hun;
#define	icmp_pptr	icmp_hun.ih_pptr
#define	icmp_gwaddr	icmp_hun.ih_gwaddr
#define	icmp_id		icmp_hun.ih_idseq.icd_id
#define	icmp_seq	icmp_hun.ih_idseq.icd_seq
#define	icmp_void	icmp_hun.ih_void
#define	icmp_pmvoid	icmp_hun.ih_pmtu.ipm_void
#define	icmp_nextmtu	icmp_hun.ih_pmtu.ipm_nextmtu
#define	icmp_num_addrs	icmp_hun.ih_rtradv.irt_num_addrs
#define	icmp_wpa	icmp_hun.ih_rtradv.irt_wpa
#define	icmp_lifetime	icmp_hun.ih_rtradv.irt_lifetime
  union
  {
    struct
    {
      u_int32_t its_otime;
      u_int32_t its_rtime;
      u_int32_t its_ttime;
    } id_ts;
    struct
    {
      struct ip idi_ip;
      /* options and then 64 bits of data */
    } id_ip;
    struct icmp_ra_addr id_radv;
    u_int32_t   id_mask;
    u_int8_t    id_data[1];
  } icmp_dun;
#define	icmp_otime	icmp_dun.id_ts.its_otime
#define	icmp_rtime	icmp_dun.id_ts.its_rtime
#define	icmp_ttime	icmp_dun.id_ts.its_ttime
#define	icmp_ip		icmp_dun.id_ip.idi_ip
#define	icmp_radv	icmp_dun.id_radv
#define	icmp_mask	icmp_dun.id_mask
#define	icmp_data	icmp_dun.id_data
};

#define	ICMP_MINLEN	8				
#define	ICMP_TSLEN	(8 + 3 * sizeof (n_time))	
#define	ICMP_MASKLEN	12				
#define	ICMP_ADVLENMIN	(8 + sizeof (struct ip) + 8)	

#define	ICMP_UNREACH		3	
#define	ICMP_SOURCEQUENCH	4	
#define	ICMP_ROUTERADVERT	9	
#define	ICMP_ROUTERSOLICIT	10	
#define	ICMP_TIMXCEED		11	
#define	ICMP_PARAMPROB		12	
#define	ICMP_TSTAMP		13	
#define	ICMP_TSTAMPREPLY	14	
#define	ICMP_IREQ		15	
#define	ICMP_IREQREPLY		16	
#define	ICMP_MASKREQ		17	
#define	ICMP_MASKREPLY		18	

#define	ICMP_MAXTYPE		18

#define	ICMP_UNREACH_NET	        0
#define	ICMP_UNREACH_HOST	        1
#define	ICMP_UNREACH_PROTOCOL	        2
#define	ICMP_UNREACH_PORT	        3
#define	ICMP_UNREACH_NEEDFRAG	        4
#define	ICMP_UNREACH_SRCFAIL	        5
#define	ICMP_UNREACH_NET_UNKNOWN        6
#define	ICMP_UNREACH_HOST_UNKNOWN       7
#define	ICMP_UNREACH_ISOLATED	        8
#define	ICMP_UNREACH_NET_PROHIB	        9
#define	ICMP_UNREACH_HOST_PROHIB        10
#define	ICMP_UNREACH_TOSNET	        11
#define	ICMP_UNREACH_TOSHOST	        12
#define	ICMP_UNREACH_FILTER_PROHIB      13
#define	ICMP_UNREACH_HOST_PRECEDENCE    14
#define	ICMP_UNREACH_PRECEDENCE_CUTOFF  15

#define	ICMP_REDIRECT_NET	0	
#define	ICMP_REDIRECT_HOST	1	
#define	ICMP_REDIRECT_TOSNET	2	
#define	ICMP_REDIRECT_TOSHOST	3		

#define	ICMP_TIMXCEED_INTRANS	0
#define	ICMP_TIMXCEED_REASS	1

#define	ICMP_PARAMPROB_OPTABSENT 1	

#define	ICMP_INFOTYPE(type) \
	((type) == ICMP_ECHOREPLY || (type) == ICMP_ECHO || \
	(type) == ICMP_ROUTERADVERT || (type) == ICMP_ROUTERSOLICIT || \
	(type) == ICMP_TSTAMP || (type) == ICMP_TSTAMPREPLY || \
	(type) == ICMP_IREQ || (type) == ICMP_IREQREPLY || \
	(type) == ICMP_MASKREQ || (type) == ICMP_MASKREPLY)

// --------------------- ## --------------------------------------

#ifdef __FAVOR_BSD

struct udphdr
{
  u_int16_t uh_sport;		/* source port */
  u_int16_t uh_dport;		/* destination port */
  u_int16_t uh_ulen;		/* udp length */
  u_int16_t uh_sum;		/* udp checksum */
};

#else

struct udphdr
{
  u_int16_t source;
  u_int16_t dest;
  u_int16_t len;
  u_int16_t check;
};
#endif

/* This is a name for the 48 bit ethernet address available on many
   systems.  */
struct ether_addr
{
  u_int8_t ether_addr_octet[ETH_ALEN];
} __attribute__ ((__packed__));

/* 10Mb/s ethernet header */
struct ether_header
{
  u_int8_t  ether_dhost[ETH_ALEN];	/* destination eth addr	*/
  u_int8_t  ether_shost[ETH_ALEN];	/* source ether addr	*/
  u_int16_t ether_type;		        /* packet type ID field	*/
} __attribute__ ((__packed__));

/* Ethernet protocol ID's */
#define	ETHERTYPE_PUP		0x0200          /* Xerox PUP */
#define	ETHERTYPE_IP		0x0800		/* IP */
#define	ETHERTYPE_ARP		0x0806		/* Address resolution */
#define	ETHERTYPE_REVARP	0x8035		/* Reverse ARP */

#define	ETHER_ADDR_LEN	ETH_ALEN                 /* size of ethernet addr */
#define	ETHER_TYPE_LEN	2                        /* bytes in type field */
#define	ETHER_CRC_LEN	4                        /* bytes in CRC field */
#define	ETHER_HDR_LEN	ETH_HLEN                 /* total octets in header */
#define	ETHER_MIN_LEN	(ETH_ZLEN + ETHER_CRC_LEN) /* min packet length */
#define	ETHER_MAX_LEN	(ETH_FRAME_LEN + ETHER_CRC_LEN) /* max packet length */

/* make sure ethenet length is valid */
#define	ETHER_IS_VALID_LEN(foo)	\
	((foo) >= ETHER_MIN_LEN && (foo) <= ETHER_MAX_LEN)

/*
 * The ETHERTYPE_NTRAILER packet types starting at ETHERTYPE_TRAIL have
 * (type-ETHERTYPE_TRAIL)*512 bytes of data followed
 * by an ETHER type (as given above) and then the (variable-length) header.
 */
#define	ETHERTYPE_TRAIL		0x1000		/* Trailer packet */
#define	ETHERTYPE_NTRAILER	16

#define	ETHERMTU	ETH_DATA_LEN
#define	ETHERMIN	(ETHER_MIN_LEN - ETHER_HDR_LEN - ETHER_CRC_LEN)

#endif	/* TCPIP_H */
